﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace WindowsFormsApplication1
{
    class _12306Class
    {
        public static CookieCollection Cookies
        {
            get { return cookies; }
        }

        static CookieCollection cookies = new CookieCollection();

        private const string UrlLoginImage = "http://dynamic.12306.cn/otsweb/passCodeAction.do?rand=sjrand";
        private const string UrlMainPage = "https://dynamic.12306.cn/otsweb/main.jsp";
        private const string UrlLoginStep1 = "http://dynamic.12306.cn/otsweb/loginAction.do?method=loginAysnSuggest";
        private const string UrlLoginStep2 = "http://dynamic.12306.cn/otsweb/loginAction.do?method=login";
        private const string UrlFavoriteContacts = "http://dynamic.12306.cn/otsweb/passengerAction.do?method=getPagePassengerAll";


        /// <summary>
        /// 获取登录图片
        /// </summary>
        /// <param name="userAgent"></param>
        /// <param name="cookie"></param>
        /// <returns></returns>
        public static Bitmap GetLoginImage(int? timeout = null, string userAgent = null, CookieCollection cookie = null)
        {
            try
            {
                var response = HttpHelper.CreateGetHttpResponse(UrlLoginImage, timeout, userAgent, cookie ?? Cookies);
                Stream resStream = response.GetResponseStream();//得到验证码数据流
                return new Bitmap(resStream);//初始化Bitmap图片
            }
            catch (Exception)
            {
                return null;
            }
        }

        /// <summary>
        /// 登录首页，获取Cookie
        /// </summary>
        /// <param name="userAgent"></param>
        /// <param name="cookie"></param>
        public static void GetMainPage(int? timeout = null, string userAgent = null, CookieCollection cookie = null)
        {
            var response = HttpHelper.CreateGetHttpResponse(UrlMainPage, timeout, userAgent, cookie ?? Cookies, "");

            cookies.Add(response.Cookies);
        }

        /// <summary>
        /// 登录
        /// </summary>
        /// <param name="userName"></param>
        /// <param name="pass"></param>
        /// <param name="verificationCode"></param>
        /// <param name="timeout"></param>
        /// <param name="userAgent"></param>
        /// <param name="cookie"></param>
        /// <returns></returns>
        public static LoginResponse Login(string userName, string pass, string verificationCode
            , int? timeout = null, string userAgent = null, CookieCollection cookie = null)
        {
            //   获取登录随机验证码loginRand
            string str = "";
            var response = HttpHelper.CreateGetHttpResponse(UrlLoginStep1, timeout, userAgent, cookie ?? Cookies, "http://dynamic.12306.cn/otsweb/loginAction.do?method=init");
            cookies.Add(response.Cookies);

            try
            {
                using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    str = reader.ReadToEnd();
                }

                if (str.Length > 50)
                {
                    return new LoginResponse() { IsLogined = false, Message = "网络异常，请稍后重试。", type = ErrorType.NetworkError };
                }
            }
            catch (Exception)
            {
                return new LoginResponse() { IsLogined = false, Message = "网络可能存在问题，请您重试一下！", type = ErrorType.NetworkError };
            }

            // 解析第一次返回值，获取登录随机验证码loginRand，执行登录
            var v = new IronJS.Hosting.CSharp.Context();
            string strJson = "var json = " + str;
            dynamic dy = v.Execute(strJson);

            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("loginUser.user_name", userName);
            dic.Add("nameErrorFocus", "");
            dic.Add("user.password", pass);
            dic.Add("passwordErrorFocus", "");
            dic.Add("randCode", verificationCode);
            dic.Add("randErrorFocus", "");
            dic.Add("loginRand", dy.loginRand);
            response = HttpHelper.CreatePostHttpResponse(UrlLoginStep2, dic, timeout, userAgent, Encoding.UTF8, cookie ?? Cookies, "http://dynamic.12306.cn/otsweb/loginAction.do?method=init");

            cookies.Add(response.Cookies);

            try
            {
                using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    str = reader.ReadToEnd();
                }
            }
            catch (Exception)
            {
                return new LoginResponse() { IsLogined = false, Message = "网络可能存在问题，请您重试一下！", type = ErrorType.NetworkError };
            }

            Regex logined = new Regex(@"'[^']*? ，您好！'");

            Regex networkError = new Regex(@"网络可能存在问题，请您重试一下");

            Regex verificationError = new Regex(@"请输入正确的验证码");

            Regex PasswordError = new Regex(@"密码输入错误");

            if (logined.IsMatch(str))
            {
                return new LoginResponse() { IsLogined = true, Message = "", type = ErrorType.None, LoginName = logined.Matches(str)[0].Value };
            }
            else if (verificationError.IsMatch(str))
            {
                return new LoginResponse() { IsLogined = false, Message = "请输入正确的验证码！", type = ErrorType.NetworkError };
            }
            else if (networkError.IsMatch(str))
            {
                return new LoginResponse() { IsLogined = false, Message = "网络可能存在问题，请您重试一下！", type = ErrorType.NetworkError };
            }
            else if (PasswordError.IsMatch(str))
            {
                return new LoginResponse() { IsLogined = false, Message = "密码错误！", type = ErrorType.PassordError };
            }

            return new LoginResponse() { IsLogined = false, Message = "未知错误！", type = ErrorType.OtherError };
        }

        /// <summary>
        /// 获取常用联系人
        /// </summary>
        /// <param name="timeout"></param>
        /// <param name="userAgent"></param>
        /// <param name="cookie"></param>
        public static List<Contact> GetFavoriteContacts(int pageIndex, int pageSize, out int count, int? timeout = null, string userAgent = null,
                                               CookieCollection cookie = null)
        {
            List<Contact> contacts = new List<Contact>();
            count = 0;
            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("pageIndex", pageIndex.ToString());
            dic.Add("pageSize", pageSize.ToString());
            dic.Add("passenger_name", "请输入汉字或拼音首字母");
            var response = HttpHelper.CreatePostHttpResponse(UrlFavoriteContacts, dic, timeout, userAgent, Encoding.UTF8, cookie ?? Cookies, "https://dynamic.12306.cn/otsweb/passengerAction.do?method=initUsualPassenger12306");

            cookies.Add(response.Cookies);
            dynamic dy;
            try
            {
                using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.UTF8))
                {
                    var str = reader.ReadToEnd();

                    var v = new IronJS.Hosting.CSharp.Context();
                    string strJson = "var json = " + str;

                    dy = v.Execute(strJson);

                    count = (int)dy.recordCount;

                    for (int i = 0; i < dy.rows.length; i++)
                    {
                        contacts.Add(new Contact()
                        {
                            Name = dy.rows[i].passenger_name,
                            IdTypeName = dy.rows[i].passenger_id_type_name,
                            IdNo = dy.rows[i].passenger_id_no,
                            IdTypeCode = dy.rows[i].passenger_id_type_code,
                            Mobile = dy.rows[i].mobile_no,
                            PassengerTypeName = dy.rows[i].passenger_type_name,
                            PassengerType = dy.rows[i].passenger_type
                        });
                    }
                }
            }
            catch (Exception)
            {

            }

            return contacts;
        }
    }

    class Contact
    {
        public string Name;
        public string IdTypeName;
        public string IdNo;
        public string IdTypeCode;
        public string Mobile;
        public string PassengerTypeName;
        public string PassengerType;
    }

    class LoginResponse
    {
        public bool IsLogined;
        public string Message;
        public string LoginName;
        public ErrorType type;
    }

    public enum ErrorType
    {
        None,
        NetworkError,
        VerificationError,
        PassordError,
        OtherError
    }
}
